@@ -5,45 +5,44 @@ use warnings;
 use Carp qw/croak/;
 use Scalar::Util qw/blessed/;
 use MooX::ReturnModifiers;
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub import {
-    my ( $self, @import ) = @_;
-
+    my ( $package, @import ) = @_;
+    
     my $target = caller;
-    my %modifiers = return_modifiers($target, [qw/has around/]);
+    my %modifiers = return_modifiers($target, [qw/has around with/]);
 
     my $variant = sub {
-        my ($name, %attributes) = @_;
+        my ($name, %spec) = @_;
+       
+        $modifiers{has}->(
+            $name => _construct_attribute($name, %spec)
+        );
 
-        my $when = {@{ delete $attributes{when} }};
-        $modifiers{has}->($name => %attributes); 
-
-        $modifiers{around}->($name => sub {
-            my ($orig, $proto ) = (shift, shift); 
-            my $new = $proto->$orig(@_); 
-            my $obj_isa = blessed $new;
-            if (my $options = $when->{$obj_isa}) {
-                for my $alias (keys %{$options->{alias}}) {
-                    next if $new->can($alias);
-                    my $actual = $options->{alias}->{$alias};
-                    {
-                        no strict 'refs';
-                        *{"${obj_isa}::${alias}"} = sub { goto &{"${obj_isa}::${actual}"} };
-                    }
-                }
-                return $new;
-            }
-            croak sprintf 'Trying to initiate attribute - %s - with something unsupported - %s - valid when - [ %s ]',
-                $name, $obj_isa, join(', ', keys %{$when});
-        });
     };
 
-    { no strict 'refs'; *{"${target}::variant"} = $variant; }
+    $target->can('_given_when') or $modifiers{with}->( 'MooX::VariantAttribute::Role' );
+
+    { no strict 'refs'; *{"${target}::variant"} = $variant;}
 
     return 1;
 }
 
+sub _construct_attribute {
+    my ($name, %spec) = @_;
+
+    my $trigger = sub {
+        return $_[0]->_given_when($_[1], $spec{given}, $spec{when}, $name);
+    };
+
+    return (
+        is => $spec{is} ? $spec{is} : 'rw',
+        trigger => $trigger,
+        lazy => 1,
+    );
+}
+
 1;
 
 __END__
@@ -54,7 +53,7 @@ MooX::VariantAttribute - a щ（ﾟДﾟщ）Attribute...
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
@@ -63,10 +62,11 @@ Version 0.02
     package My::Multi::Parser
     use Moo;
     use MooX::VariantAttribute;
+    use Type::Standards qw/Obj/;
 
     # variant accepts everything - has - does
     variant parser => (
-        is  => 'ro',
+        given => Obj,
         when => [
             'Test::Parser::One' => {
                 alias => {
@@ -89,6 +89,26 @@ Version 0.02
         ],
     );
 
+    variant refs => (
+        given => sub { ref $_[1] or ref \$_[1] }, 
+        when => [
+            'SCALAR' => { 
+                run => sub { return "refs returned - SCALAR - $_[1]" },
+            },
+            'HASH' => {
+                run => sub { 
+                    return "refs returned - HASH - " . 
+                        join ',', map { sprintf '%s=>%s', $_, $_[1]->{$_} } keys %{ $_[1] }; 
+                },
+            },
+            'ARRAY' => {
+                run => sub { 
+                    return "refs returned - ARRAY - " . join ',', @{ $_[1] } 
+                },
+            },
+        ],
+    );
+
     ........
 
     my $parser = My::Multi::Parser->new( parser => Another::Parser::Three->new() )->parser;
